//
//  MINTLMigemodokiDictionary.swift
//  TextEditDoc
//
//  Created by narita on 2019/10/06.
//  Copyright © 2019 narita. All rights reserved.
//

import Cocoa

// 
class MINTLMigemodokiDictionary: NSObject
{
    //
    typealias MINTLMigemodokiRecode = (inputString: String, outputStrings: Array<String>)
    typealias Index = Array<MINTLMigemodokiRecode>.Index

    // MARK: - class methods
    
    // MARK: - init methods
    
    override init()
    {
        super.init()
        
        // サーチ済み配列の作成
        self.sortedTransformArray = self.transformArray.sorted(by: { $0.inputString > $1.inputString})
                
    }
    
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods
    
    // 辞書にある定義を複数取り出す
    @objc func titles(for searchString: NSString) -> NSArray
    {        
        let theSetString = binarySearchTransform(input: searchString as String)

        return theSetString as NSArray
    }
    
    // 二分検索を使ったサーチ
    func binarySearchTransform(input: String) -> Array<String>
    {
        // 両方ともnullならば、空集合を返す。
        if input.count == 0
        {
            return []
        }

        // 最初の行を探す
        func findFirst(searchString: String,
                       array: Array<MINTLMigemodokiRecode>) -> Index
        {
            var theStart:  Index = array.startIndex
            var theEnd:    Index = array.endIndex
            var theMiddle: Index = 0
            var theResult: Index = 0
            
            let lhs = searchString

            while theStart < theEnd
            {
                theMiddle = theStart.advanced(by: theStart.distance(to: theEnd) / 2)

                let rhs = array[theMiddle].inputString
                                
                if rhs.hasPrefix(lhs)
                {
                    theResult = theMiddle
                    theEnd = theMiddle
                }
                else if lhs < rhs
                {
                    theStart = theMiddle + 1
                }
                else
                {
                    theEnd = theMiddle
                }
            }

            return theResult
        }
        
        // 最後の行を探す
        func findLast(searchString: String,
                     array: Array<MINTLMigemodokiRecode>) -> Index
        {
            var theStart:  Index = array.startIndex
            var theEnd:    Index = array.endIndex
            var theMiddle: Index = 0
            var theResult: Index = 0

            let lhs = searchString

            while theStart < theEnd
            {
                theMiddle = theStart.advanced(by: theStart.distance(to: theEnd) / 2)

                let rhs = array[theMiddle].inputString

                if rhs.hasPrefix(lhs)
                {
                    theResult = theMiddle
                    theStart = theMiddle + 1
                }
                else if lhs < rhs
                {
                    theStart = theMiddle + 1
                }
                else
                {
                    theEnd = theMiddle
                }
            }

            return theResult
        }

        let theFirst = findFirst(searchString: input, array:self.sortedTransformArray)
        let theLast  = findLast(searchString: input, array:self.sortedTransformArray)
        
        // 両方ともnullならば、空集合を返す。
        if theFirst == 0 && theLast == 0
        {
            return []
        }

        // それぞれの集合を返す。
        var theResult: Array<String> = []
        
        self.sortedTransformArray[ theFirst ... theLast ].forEach()
        {
            theResult.append(contentsOf: $0.outputStrings)
        }        
        
        return theResult
    }
    
    // MARK: - event handling methods
    
    // MARK: - drawing methods
    
    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    // 返値はArray<(inputString: String, outputStrings: set<String>)>
    
    let transformArray: Array<(inputString: String, outputStrings: Array<String>)> =
    {
        let theURL = Bundle.main.url(forResource: "migemo-dict", withExtension:"txt")
        
        let theRomajiRuleTxt = try! NSString(contentsOf: theURL!,
                                             encoding: String.Encoding.utf8.rawValue)
        
        let theLines = theRomajiRuleTxt.components(separatedBy: CharacterSet.newlines)
        
        let theTransformArray: Array<(inputString: String, outputStrings: Array<String>)> =
            theLines
                .filter()// "#"で始まる行をコメント行として除外
                {
                    $0.hasPrefix("#") == false
                }
                .compactMap()
                {
                    let theSeparetedLine = $0.components(separatedBy: CharacterSet.whitespaces)
                        
                    return ( inputString: theSeparetedLine.first!,
                         outputStrings: Array<String>(theSeparetedLine.dropFirst()))
                }
        
        return theTransformArray
    }()
        
    var sortedTransformArray: Array<MINTLMigemodokiRecode> = []
    
    // MARK: - Utility methods

}
