//
//  NSString+Henkan.swift
//  TextEditDoc
//
//  Created by narita on 2019/09/05.
//  Copyright © 2019 narita. All rights reserved.
//
// http://ch3cooh.jp/ にある　NSString+Henkan.m を参考にした。
// ここを見ないと、引数reverseがtrue/falseが意味不明すぎてわからない。

import Foundation

extension NSString // Henkan
{
    private func stringTransform(with transform: CFString, reverse: Bool) -> NSString?
    {
        let theResult = NSMutableString(string: self)
  
        CFStringTransform(theResult as CFMutableString, nil, transform, reverse)
        
        return theResult
    }
    
    // 半角→全角
    @objc func stringToFullwidth() -> NSString?
    {
        return stringTransform(with: kCFStringTransformFullwidthHalfwidth,
                                    reverse: true)
    }
    
    // 全角→半角
    @objc func stringToHalfwidth() -> NSString?
    {
        return stringTransform(with: kCFStringTransformFullwidthHalfwidth,
                               reverse: false)
    }

    // カタカナ→ひらがな
    @objc func stringKatakanaToHiragana() -> NSString?
    {
        return stringTransform(with: kCFStringTransformHiraganaKatakana,
                               reverse: true)
    }

    // ひらがな→カタカナ
    @objc func stringHiraganaToKatakana() -> NSString?
    {
        return stringTransform(with: kCFStringTransformHiraganaKatakana,
                               reverse: false)
    }

    // ひらがな→ローマ字
    @objc func stringHiraganaToLatin() -> NSString?
    {
        return stringTransform(with: kCFStringTransformLatinHiragana,
                               reverse: true)
    }

    // ローマ字→ひらがな
    @objc func stringLatinToHiragana() -> NSString?
    {
        return stringTransform(with: kCFStringTransformLatinHiragana,
                               reverse: false)
    }

    // カタカナ→ローマ字
    @objc func stringKatakanaToLatin() -> NSString?
    {
        return stringTransform(with: kCFStringTransformLatinKatakana,
                               reverse: true)
    }

    // ローマ字→カタカナ
    @objc func stringLatinToKatakana() -> NSString?
    {
        return stringTransform(with: kCFStringTransformLatinKatakana,
                               reverse: false)
    }
}
