//
//  TEMViewController.swift
//  TextEditMM
//
//  Created by narita on 2019/10/24.
//  Copyright © 2019 narita. All rights reserved.
//

import Cocoa

class TEMViewController: NSViewController, NSTextFieldDelegate
{
    // MARK: - class methods
    
    // MARK: - init methods
    
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods

    override func viewDidLoad()
    {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        
        // 変換中も呼び出す
        (findTextField.cell as! NSTextFieldCell).setWantsNotificationForMarkedText(true)
    }

    // MARK: - action methods
    
    // ^s: 順検索
    @IBAction func searchNext(_ sender: Any?)
    {
        if isResponder(self.findTextField)
        {
            if findTextField.stringValue == ""
            {
                
            }
            else
            {
                self.findNext(nil)
            }
        }
        else
        {
            // 現在の選択範囲を保存
            startRange = self.textView.selectedRange()
            
            self.view.window?.makeFirstResponder(self.findTextField)
            updateMatchRanges(true)
        }
    }
    
    // ^r: 逆検索
    @IBAction func searchPrev(_ sender: Any?)
    {
        if isResponder(self.findTextField)
        {
            if findTextField.stringValue == ""
            {
                
            }
            else
            {
                self.findPrev(nil)
            }
        }
        else
        {
            // 現在の選択範囲を保存
            startRange = self.textView.selectedRange()

            self.view.window?.makeFirstResponder(self.findTextField)
            updateMatchRanges(false)
        }
    }

    // ^m: 選択位置を決定、textViewへFocusを移す
    @IBAction func searchResult(_ sender: Any?)
    {
        textView.selectedRange = currentRange
        currentRange = NSMakeRange(0, 0)
        
        self.view.window?.makeFirstResponder(self.textView)
    }
    
    @IBAction func updateMatchRanges(_ order: Bool)
    {
        let theReg = regularExp(string: findTextField.stringValue)
        
        var theSelectedRange: NSArray = []
        
        if theReg != nil
        {
            theSelectedRange = (theReg!.matches(in: textView.string,
                                               options: [],
                                               range: NSMakeRange(0, textView.string.count)) as NSArray).value(forKey:"range") as! NSArray
        }
        
        // 必要ならば注目している場所(黄色いピョン)の位置の修正
        if theSelectedRange.contains( NSValue(range: self.currentRange) ) == false
        {
            // 最初の選択範囲を選ぶ
            if theSelectedRange.count != 0
            {
                // 検索を開始したカーソル位置
                let theStart = startRange.location

                if order
                {
                    // 開始したカーソル位置から、順方向へ向かって最初に見つかった文字列を選択する
                    if let theElement = theSelectedRange.first(where:{ ($0 as! NSValue).rangeValue.location >= theStart })
                    {
                        currentRange = (theElement as! NSValue).rangeValue
                    }
                    else
                    {
                        // 最後まで見つからない場合は、ドキュメントの最初の位置に戻る
                        currentRange = (theSelectedRange.firstObject as! NSValue).rangeValue
                    }
                }
                else
                {
                    // 開始したカーソル位置から、逆方向へ向かって最初に見つかった文字列を選択する
                    if let theElement = theSelectedRange.reversed().first(where:{ ($0 as! NSValue).rangeValue.location < theStart })
                    {
                        currentRange = (theElement as! NSValue).rangeValue
                    }
                    else
                    {
                        // 最後まで見つからない場合は、ドキュメントの最後の位置に戻る
                        currentRange = (theSelectedRange.lastObject as! NSValue).rangeValue
                    }

                }

                currentRange = (theSelectedRange[0] as! NSValue).rangeValue
            }
            else
            {
                //元の選択範囲の先頭にする
                currentRange = NSMakeRange(startRange.location, 0)
            }
        }
        
        // 選択範囲群の再設定
        if theSelectedRange.count != 0
        {
            textView.selectedRanges = theSelectedRange as! [NSValue]
        }
        else
        {
            //元の選択範囲の先頭にする
            textView.selectedRange = NSMakeRange(startRange.location, 0)
        }
        
        self.scrollToVisibleAfterShowAfterIndicatorForCurrentRange(self)
    }

    @IBAction func findNext(_ sender: Any?)
    {
        if currentRange.length != 0
        {
            var theIndex: NSInteger?
                        
            theIndex = textView.selectedRanges.firstIndex(of: NSValue(range: self.currentRange))
            
            if theIndex != nil
            {
                // 一個前に進める
                theIndex! += 1
                
                // 最後までいったら最初に戻す
                if theIndex! >= textView.selectedRanges.count
                {
                    theIndex! = 0
                }
                
                currentRange = textView.selectedRanges[theIndex!].rangeValue
                
                self.scrollToVisibleAfterShowAfterIndicatorForCurrentRange(self)
            }
        }
    }

    @IBAction func findPrev(_ sender: Any?)
    {
        if currentRange.length != 0
        {
            var theIndex: NSInteger?
                        
            theIndex = textView.selectedRanges.firstIndex(of: NSValue(range: self.currentRange))
            
            if theIndex != nil
            {
                // 一個後ろに進める
                theIndex! -= 1
                
                //  最初までいったら最後に戻す
                if theIndex! < 0
                {
                    theIndex! = textView.selectedRanges.count - 1
                }
                
                currentRange = textView.selectedRanges[theIndex!].rangeValue
                
                self.scrollToVisibleAfterShowAfterIndicatorForCurrentRange(self)
            }
        }
    }

    @IBAction func scrollToVisibleAfterShowAfterIndicatorForCurrentRange(_ sender: Any?)
    {
        textView.scrollRangeToVisible(currentRange)
        
        textView.showFindIndicator(for: currentRange)
    }
    
    
    // MARK: - event handling methods

    // MARK: - drawing methods
    
    // MARK: - delegate/datasource methods

    // MARK: - NSTextFieldDelegate methods
    
    func controlTextDidChange(_ obj: Notification)
    {
        if findTextField == obj.object as? NSTextField
        {
            updateMatchRanges(true)
        }
    }
    
    // MARK: - accessor methods (in pairs)
    
    @IBOutlet var textView: NSTextView! = nil

    @IBOutlet var findTextField: NSTextField! = nil
    
    var currentRange: NSRange = NSRange()
    var startRange: NSRange = NSRange()
    
    override var representedObject: Any?
    {
        didSet
        {
            // Update the view, if already loaded.
            if let theDocument = representedObject as! TEMDocument?
            {
                self.textView!.layoutManager!.replaceTextStorage(theDocument.textStorage!)
            }
        }
    }
    
    // MARK: - Utility methods

    

    func regularExp(string: String)-> NSRegularExpression?
    {
        
        // こっちを有効にすると単純なgrepになる。
//        return try? NSRegularExpression(pattern: NSRegularExpression.escapedPattern(for: string),
//                                        options: [])
        
                
        if let theAppDelegate = NSApp.delegate as? TEMAppDelegate
        {
            return theAppDelegate.migemodoki.regularExpression(inputString: string as NSString)
        }

        return nil
    }
    
    //         firstResponderからnextResponderを降って存在するか？
    func isResponder(_ responder: NSResponder) -> Bool
    {
        var theResult = false
        var theResponder = self.view.window?.firstResponder
        while( true)
        {
            if theResponder == responder
            {
                theResult = true
                break
            }

            if theResponder == nil || theResponder == self.view.window
            {
                break
            }
            
            theResponder = theResponder?.nextResponder
        }
        
        return theResult
    }
}

